package com.zheng.demo.mapper;

import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.function.Function;

import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import com.zheng.demo.data.Car;
import com.zheng.demo.data.CarDto;
import com.zheng.demo.data.Train;
import com.zheng.demo.data.Vehicle;
import com.zheng.demo.data.VehicleDto;

class TestHigherOrderFunction {

	Function<CarDto, Car> carDtoToCarMap = (c -> {
		return Mappers.getMapper(CarMapper.class).carDtoToCar(c);
	});

	Function<Car, CarDto> carToCarDtoMap = (c -> {
		return Mappers.getMapper(CarMapper.class).carToCarDto(c);
	});

	Function<Vehicle, VehicleDto> vehicleDtoToCarMap = (c -> {
		if (c instanceof Car) {
			return (VehicleDto) Mappers.getMapper(CarMapper.class).carToCarDto((Car) c);
		} else if (c instanceof Train) {
			return (VehicleDto) Mappers.getMapper(TrainMapper.class).trainToTrainDto((Train) c);
		} else {
			return null;
		}
	});

	private Car applyCarDtoToCarMap(CarDto carDto, Function<CarDto, Car> function) {
		return function.apply(carDto);
	}

	private CarDto applyCarToCarDtoMap(Car car, Function<Car, CarDto> function) {
		return function.apply(car);
	}

	private VehicleDto applyToDtoMap(Vehicle car, Function<Vehicle, VehicleDto> function) {
		return function.apply(car);
	}

	@Test
	void test_map_via_high_order_functions() {
		Car car = new Car("Morris", 5, "SEDAN");
		car.setColor("white");
		car.setSpeed("50mph");

		CarDto cd = applyCarToCarDtoMap(car, carToCarDtoMap);
		assertTrue(cd instanceof CarDto);

		Car car2 = applyCarDtoToCarMap(cd, carDtoToCarMap);
		assertTrue(car2 instanceof Car);

		VehicleDto carV = applyToDtoMap(car, vehicleDtoToCarMap);
		assertTrue(carV instanceof CarDto);
	}

}
