package com.zheng.demo.mapper;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;

import com.zheng.demo.data.Car;
import com.zheng.demo.data.CarDto;
import com.zheng.demo.data.Train;
import com.zheng.demo.data.TrainDto;
import com.zheng.demo.data.Vehicle;
import com.zheng.demo.data.VehicleDto;

class VehicleMapper_SubclassMappingTest {

	private VehicleMapper_SubclassMapping mapper = new VehicleMapper_SubclassMappingImpl();

	@Test
	void shouldMapParentAndChildFields_car() {
		CarDto carDto = CarDto.builder().seatCount(6).color("white").make("Chevrolet").type("Malibu").speed("50mph")
				.build();
		Car car = (Car) mapper.dtoToVehicle(carDto);

		assertThat(car).isNotNull();
		assertThat(car.getMake()).isEqualTo(carDto.getMake());
		assertThat(car.getNumberOfSeats()).isEqualTo(carDto.getSeatCount());
		assertThat(car.getType()).isEqualTo(carDto.getType());
		assertThat(car.getColor()).isEqualTo(carDto.getColor());
		assertThat(car.getSpeed()).isEqualTo(carDto.getSpeed());

		// map back to CarDto
		CarDto carDto2 = (CarDto) mapper.vehicleToDto(car);

		assertThat(carDto2).isNotNull();
		assertThat(carDto2.getMake()).isEqualTo(car.getMake());
		assertThat(carDto2.getSeatCount()).isEqualTo(car.getNumberOfSeats());
		assertThat(carDto2.getType()).isEqualTo(car.getType());

		// super class
		assertThat(carDto2.getColor()).isEqualTo(car.getColor());
		assertThat(carDto2.getSpeed()).isEqualTo(car.getSpeed());

	}

	@Test
	void shouldMapParentAndChildFields_train() {
		TrainDto trainDto = TrainDto.builder().numberOfKarts(6).color("white").speed("50mph").build();

		Vehicle train = mapper.dtoToVehicle(trainDto);

		assertThat(train).isNotNull();
		assertTrue(train instanceof Train);
		assertThat(train.getColor()).isEqualTo(trainDto.getColor());
		assertThat(train.getSpeed()).isEqualTo(trainDto.getSpeed());

		// map back to CarDto
		VehicleDto trainDto2 = mapper.vehicleToDto(train);

		assertTrue(trainDto2 instanceof TrainDto);
		assertThat(trainDto2).isNotNull();
		assertThat(trainDto2.getColor()).isEqualTo(train.getColor());
		assertThat(trainDto2.getSpeed()).isEqualTo(train.getSpeed());
	}

}
